/* StockDataSource.m created by mlaster on Thu 15-Jul-1999 */

#import "StockDataSource.h"
#import "NSBundle+BundleSearching.h"
#import "PropertyListCategories.h"
#import "NSNumber+FractionExtensions.h"

static NSMutableArray *dataSourceClassNames = nil;

@implementation StockDataSource

+ (StockDataSource *) dataSource
{
    StockDataSource *newInstance = nil;

    if ([dataSourceClassNames count] > 0)
    {
        newInstance = [[[NSClassFromString([dataSourceClassNames objectAtIndex:1]) alloc] init] autorelease];
    }
    return newInstance;
}

+ (void) loadDataSourceBundles
{
    NSArray *bundleArray = nil;
    unsigned i=0;
    
    bundleArray = [NSBundle availableBundlesOfType:@"StockDataSource"];
    for (i=0;i<[bundleArray count];i++)
    {
        if ([[bundleArray objectAtIndex:i] smartLoad] == NO)
        {
            NSLog(@"Unable to load [%@]", [bundleArray objectAtIndex:i]);
        }
    }
}

+ (void) registerDataSource:(NSString *)className
{
    if (dataSourceClassNames == nil)
    {
        dataSourceClassNames = [[NSMutableArray alloc] init];
    }
    [dataSourceClassNames addObject:className];
}

- (id) init
{
    NSMutableDictionary *newDictionary = nil;
    
    self = [super init];
    if (self != nil)
    {
        newDictionary = [[NSMutableDictionary alloc] init];
        [self setStatDictionary:newDictionary];
        colorCodes = [[NSDictionary alloc] initWithObjectsAndKeys:
            [NSColor whiteColor], @"Neutral",
            [NSColor greenColor], @"Positive",
            [NSColor redColor], @"Negative",
            nil,nil];
        [newDictionary release];
    }
    return self;
}

- (void) dealloc
{
    [colorCodes release];
    [self setStatDictionary:nil];
    [super dealloc];
}
- (void) update
{
    NSDictionary *tempDictionary = nil;
    
    // synchronize preferences, and notify everyone that the data has been updated
    tempDictionary = [[statDictionary plist] copy];
    [[NSUserDefaults standardUserDefaults] setObject:tempDictionary forKey:@"Symbols"];
    [tempDictionary release];
    [[NSUserDefaults standardUserDefaults] synchronize];
    [[NSNotificationCenter defaultCenter] postNotificationName:StockDataUpdatedNotification
                                                        object:self
                                                      userInfo:nil];
}

// symbol list management
- (void) addSymbol:(NSMutableDictionary *)aSymbol
{
    NSString *key = nil;

    key = [aSymbol objectForKey:@"Symbol"];
    NSAssert(key != nil, @"addSymbol: dictionary must contain 'Symbol' key!");
    if ([[self statDictionary] objectForKey:key] == nil)
    {
        [[self statDictionary] setObject:aSymbol forKey:key];
    }
}

- (void) removeSymbol:(NSString *)aSymbol
{
    [[self statDictionary] removeObjectForKey:aSymbol];
}

- (void) removeAllSymbols
{
    [[self statDictionary] removeAllObjects];
}

- (NSAttributedString *) attributedString
{
    unsigned i=0;
    NSMutableAttributedString *workString = nil;
    NSArray *keys = nil;
    NSMutableDictionary *stock = nil;

    keys = [[[self statDictionary] allKeys] sortedArrayUsingSelector:@selector(compare:)];
    workString = [[[NSMutableAttributedString alloc] initWithString:@""] autorelease];

    for (i=0;i<[keys count];i++)
    {
        stock = [[self statDictionary] objectForKey:[keys objectAtIndex:i]];
        if (i != 0)
        {
            [[workString mutableString] appendString:@"   "];
        }
        [workString appendAttributedString:[self attributedStringForStockDictionary:stock]];
    }
    return workString;
}

- (NSAttributedString *) attributedStringForStockDictionary:(NSDictionary *)aDictionary
{
    NSMutableAttributedString *workString = nil;
    NSAttributedString *fraction = nil;
    NSNumber *workNumber = nil;
    NSNumberFormatter *formatter = nil;
    NSColor *highlightColor = nil;
    double percentChange = 0.0;
    
    workString = [[NSMutableAttributedString alloc] initWithString:@""];
    // Append symbol
    [[workString mutableString] appendString:[NSString stringWithFormat:@"%@ ",
        [aDictionary objectForKey:@"Symbol"]]];

    // append price
    workNumber = [aDictionary objectForKey:@"Price"];
    NSAssert([workNumber isKindOfClass:[NSNumber class]], @"workNumber is not an NSNumber!");
    fraction=[workNumber fractionalRepresentation];
    [workString appendAttributedString:fraction];

    // append change
    [[workString mutableString] appendString:@" "];
    workNumber = [aDictionary objectForKey:@"Change"];
    NSAssert([workNumber isKindOfClass:[NSNumber class]], @"Change is not an NSNumber!");
    fraction=[workNumber signedFractionalRepresentation];
    [workString appendAttributedString:fraction];

    // append percent change
    percentChange = 100.0 * [[aDictionary objectForKey:@"Change"] doubleValue] /
        [[aDictionary objectForKey:@"Price"] doubleValue] ;
        [[workString mutableString] appendString:[NSString stringWithFormat:@" (%+.2f%%)", percentChange]];
    
    // append volume
    [[workString mutableString] appendString:@" "];
    [workString unscriptRange:NSMakeRange([workString length]-1,1)];
    formatter = [[NSNumberFormatter alloc] init];
    [formatter setFormat:@",0"];
    [[workString mutableString] appendString:[NSString stringWithFormat:@"Vol: %@",
        [formatter stringForObjectValue:[aDictionary objectForKey:@"Volume"]]]];
    [formatter release];
    
    // colorize entire string
    // TO-DO: make the thresholds configurable
    if (percentChange >= 10.0)
    {
        highlightColor = [colorCodes objectForKey:@"Positive"];
    }
    else if (percentChange <= -10.0)
    {
        highlightColor = [colorCodes objectForKey:@"Negative"];
    }
    else
    {
        highlightColor = [colorCodes objectForKey:@"Neutral"];
    }
    
    [workString addAttribute:NSForegroundColorAttributeName
                       value:highlightColor
                       range:NSMakeRange(0,[workString length])];

    return [workString autorelease];
}

- (NSMutableDictionary *) statDictionary
{
    return statDictionary;
}

- (void) setStatDictionary:(NSMutableDictionary *)newDictionary
{
    [newDictionary retain];
    [statDictionary release];
    statDictionary = newDictionary;
}

// this method is ugly.  There has to be a more elegant way to account
// for the fact that our NSNumbers turn back into NSStrings when we
// read them back from defaults

- (void) reloadSymbols
{
    int i=0;
    NSArray *keys = nil;
    NSString *key = nil;
    NSMutableArray *keysToRemove = nil;
    NSDictionary *symbols = nil;
    NSMutableDictionary *entity = nil;
    id workObject = nil;
    NSNumber *workNumber = nil;

    symbols = [[NSUserDefaults standardUserDefaults] dictionaryForKey:@"Symbols"];
    keys= [symbols allKeys];
    keysToRemove = [[[self statDictionary] allKeys] mutableCopy];
    for (i=0;i<[keys count];i++)
    {
        key = [keys objectAtIndex:i];
        entity = [[symbols objectForKey:key] mutableCopy];

        // objectForKey tends to return NSStrings, even when they should probably be NSNumbers...

        // convert price to an NSNumber
        workObject = [entity objectForKey:@"Price"];
        if ([workObject isKindOfClass:[NSString class]])
        {
            workNumber = [NSNumber numberWithFloat:[workObject floatValue]];
            [entity setObject:workNumber forKey:@"Price"];
        }

        // convert volume to an NSNumber
        workObject = [entity objectForKey:@"Volume"];
        if ([workObject isKindOfClass:[NSString class]])
        {
            workNumber = [NSNumber numberWithInt:[workObject intValue]];
            [entity setObject:workNumber forKey:@"Volume"];
        }

        [self addSymbol:entity];

        if ([[self statDictionary] objectForKey:key] != nil)
        {
            [keysToRemove removeObject:key];
        }
        [entity release];
    }
    for (i=0;i<[keysToRemove count];i++)
    {
        [self removeSymbol:[keysToRemove objectAtIndex:i]];
    }
    [keysToRemove release];
    [self update];
}

@end
