#import "WindowController.h"
#import "MLMarqueeView.h"
#import "StockDataSource.h"

@implementation WindowController

- (void) reloadSymbols
{
    [dataSource reloadSymbols];
}

- (StockDataSource *) dataSource
{
    return dataSource;
}

- (void) _dataUpdated:(NSNotification *)aNotification
{
    [marqueeView setAttributedString:[dataSource attributedString]];
}
- (void) _updateStockData
{
    [dataSource update];
}

- (void) _updateTick:(NSTimer *)aTimer
{
    [self _updateStockData];
    [NSTimer scheduledTimerWithTimeInterval:[aTimer timeInterval]
                                     target:self
                                   selector:_cmd
                                   userInfo:[aTimer userInfo]
                                    repeats:NO];
}

// prevent vertical resizing
- (NSSize)windowWillResize:(NSWindow *)sender toSize:(NSSize)proposedFrameSize
{
    return NSMakeSize(proposedFrameSize.width,[sender frame].size.height);
}

- (void) dealloc
{
    [dataSource release];
    [super dealloc];
}

- (void) awakeFromNib
{
    if ([[marqueeView window] setFrameAutosaveName:@"TickerWindow"] == NO)
    {
        [NSException raise:NSGenericException format:@"'TickerWindow' frame name already in use!"];
    }
    dataSource = [[StockDataSource dataSource] retain];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(_dataUpdated:)
                                                 name:StockDataUpdatedNotification
                                               object:nil];
    [[marqueeView window] setLevel:NSFloatingWindowLevel];

    // update stock quotes once a minute
    [NSTimer scheduledTimerWithTimeInterval:60.0
                                     target:self
                                   selector:@selector(_updateTick:)
                                   userInfo:nil
                                    repeats:NO];

}

@end
